var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var Vue = null;

var CacheState = {
  UNLOAD: 0,
  LOADING: 1,
  DONE: 2,
  FAILED: 3
};

function VueCacheData() {
  if (!VueCacheData.installed) {
    throw 'Please use Vue.use() install first.';
  }

  var _mixins = [];
  var _vm = null;
  this._loader = {};

  Object.defineProperty(this, 'mixins', {
    get: function get() {
      return _mixins;
    },
    set: function set(val) {
      _mixins = val;
    },
    writeable: false,
    configurable: false
  });

  Object.defineProperty(this, 'vm', {
    get: function get() {
      if (_vm === null) {
        throw new Error('VueCacheData need initialize before use');
      }
      return _vm;
    },
    set: function set(val) {
      _vm = val;
    },
    writeable: true,
    configurable: false
  });
}

VueCacheData.prototype = {
  refresh: function refresh(key, cb, retryTimes) {
    var vm = this.vm;
    var cache = vm['cache-' + key];

    if (cache.state) {
      cache.state = CacheState.UNLOAD;
      this.fetch(key, cb, retryTimes);
    } else {
      if (cb) cb(true, cache.value);
    }
  },
  fetch: function fetch(key, cb, retryTimes) {
    var _this = this;

    if (isNaN(retryTimes)) retryTimes = 1;

    var loader = this._loader[key];
    var vm = this.vm;
    var cache = vm['cache-' + key];

    switch (cache.state) {
      case CacheState.FAILED:
        if (retryTimes) {
          cache.state = CacheState.UNLOAD;
          this.fetch(key, cb, retryTimes);
        } else {
          cb && cb(false, cache.value);
        }
        break;
      case CacheState.UNLOAD:
        cache.state = CacheState.LOADING;

        if (cb) cache.cbs.push(cb);

        loader(function (val) {
          cache.value = val;
          cache.state = CacheState.DONE;

          var cbs = cache.cbs;
          cache.cbs = [];
          cbs.forEach(function (cb) {
            return cb(true, val);
          });
        }, function () {
          if (retryTimes) {
            cache.state = CacheState.UNLOAD;
            _this.fetch(key, null, --retryTimes);
          } else {
            cache.cbs = [];
            cache.state = CacheState.FAILED;
            cb && cb(false, cache.value);
          }
        });
        break;
      case CacheState.LOADING:
        if (cb) cache.cbs.push(cb);
        break;
      case CacheState.DONE:
        cb && cb(true, cache.value);
        break;
      case undefined:
        cb && cb(true, cache.value);
        break;
    }
  },
  fetchAll: function fetchAll(keys, cb, retryTimes) {
    var _this2 = this;

    var count = keys.length;
    var ret = [];

    var cancel = false;
    keys.forEach(function (key) {
      if (cancel) return;

      _this2.fetch(key, function (succeed, val) {
        if (cancel) return;

        if (!succeed) {
          cancel = true;
          cb && cb.apply(undefined, [false].concat(ret));
        }

        ret.push(val);
        count--;
        if (count === 0) {
          cb && cb.apply(undefined, [true].concat(ret));
        }
      }, retryTimes);
    });
  },
  set: function set(key, val) {
    this.vm['cache-' + key].value = val;
  },
  get: function get(key) {
    return this.vm['cache-' + key].value;
  },
  add: function add(key) {
    var defaultVal = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var loader = arguments[2];

    var mixin = {};
    var self = this;

    mixin.data = function () {
      var isFn = typeof defaultVal === 'function';
      var value = isFn ? defaultVal() : defaultVal;
      var cacheKey = 'cache-' + key;

      var ret = defineProperty({}, cacheKey, {
        value: value
      });

      if (loader) {
        Object.assign(ret[cacheKey], {
          state: CacheState.UNLOAD,
          cbs: []
        });
      }

      return ret;
    };

    if (loader) {
      self._loader[key] = loader;
      mixin.computed = defineProperty({}, key, function () {
        var dataPath = 'cache-' + key;
        if (!this.hasOwnProperty(dataPath)) return;

        var cache = this[dataPath];

        // load data
        if (cache.state === CacheState.UNLOAD) {
          self.fetch(key);
        }

        return cache.value;
      });
    }

    this.mixins.push(mixin);
  },
  init: function init() {
    this.vm = new Vue({
      mixins: this.mixins
    });
  }
};

VueCacheData.install = function (vue) {
  VueCacheData.installed = true;
  Vue = vue;
};

export default VueCacheData;
//# sourceMappingURL=vue-cache-data.mjs.map
